#!/usr/bin/python

# (c) 2006 by Markus Ullmann
# License: GPL-2

import os, os.path, sys, time

from PyQt4 import QtCore
from pkgcore.restrictions import packages, values, restriction
from pkgcore.config import load_config
from pkgcore.ebuild import atom, repository
from snakeoil import lists

try:
    from pysqlite2 import dbapi2 as sqlite
except:
    try:
	import sqlite3 as sqlite
    except:
	print "Please install sqlite3 or pysqlite2 module"
	sys.exit(1)
	
pjoin = os.path.join

class RevdepDB:
    def __init__(self):
        self.db = None

    def setDB(self,filename):
        if os.path.isfile(filename):
            os.remove(filename)
        self.db = sqlite.connect(filename)
        self.cursor = self.db.cursor()
        self.createStructure()
        self.db.commit()
           
    def createStructure(self):
        cursor = self.cursor
        cursor.execute("""CREATE TABLE rdepends (
                          atom VARCHAR(255) PRIMARY KEY,
                          rdependlist TEXT)""")

    def buildEntry(self,atom,rdepends):
        cursor = self.cursor
        cursor.execute("SELECT rdependlist FROM rdepends WHERE atom='%s'" % atom)
        if cursor.rowcount == 1:
            cursor.execute("""UPDATE rdepends WHERE atom='%s'
                              SET rdependlist='%s'""" % (atom,rdepends))
        else:
            cursor.execute("""INSERT INTO rdepends
                              ('atom','rdependlist')
                              VALUES('%s','%s')""" % (atom,rdepends))

    def commit(self):
        self.db.commit()

class RevdepBuilder(QtCore.QThread):
    def __init__(self):
        QtCore.QThread.__init__(self)
        config = load_config()
        self.reponames = []
        self.reponumber = 0
        for name,repo in config.repo.iteritems():
            if isinstance(repo, repository.UnconfiguredTree):
                self.reponames.append("%s (%s)" % (repo.repo_id, repo.location))

    def run(self):
        config = load_config()
    	repolist = []
        for name,repo in config.repo.iteritems():
            if isinstance(repo, repository.UnconfiguredTree):
                repolist.append(repo)
    	workrepo = repolist[self.reponumber]

        db = RevdepDB()
        db.setDB(self.dbfile)
        # keep the gui going
        self.setPriority(QtCore.QThread.LowPriority)

        pkgset = []
        self.emit(QtCore.SIGNAL("stepUpdate(int)"), 2)
        pkgs = workrepo.itermatch(packages.AlwaysTrue)
        for pkg in pkgs:
            pkgset.append(pkg)

        self.emit(QtCore.SIGNAL("stepUpdate(int)"), 3)
        revdepdict = {}
        for pkg in workrepo.versions:
            cpkey = "%s/%s" % pkg
            revdepdict[cpkey] = [] 

        self.emit(QtCore.SIGNAL("stepUpdate(int)"), 4)
        self.emit(QtCore.SIGNAL("setMaxCount(int)"), len(pkgset))
        for pkgno, pkg in enumerate(pkgset):
            revdeplist = []
            for field in (pkg.depends, pkg.rdepends, pkg.post_rdepends):
                for item in lists.iflatten_instance(field, atom.atom):
                    key = item.key
                    if key not in revdeplist:
                        revdeplist.append(key)

            for revdep in revdeplist:
                key = pkg.cpvstr
                if revdep in revdepdict and key not in revdepdict[revdep]:
                    revdepdict[revdep].append(key)

            self.emit(QtCore.SIGNAL("setCount(int)"), pkgno + 1)
        for k, v in revdepdict.iteritems():
            revdepstring = ""
            for revdep in v:
                revdepstring += "%s, " % revdep
            revdepstring = revdepstring[:-2]
            db.buildEntry(k,revdepstring)
        db.commit()
        self.emit(QtCore.SIGNAL("stepUpdate(int)"), 5)

    def getRepoList(self):
        return self.reponames

    def setRepo(self,reponumber):
        self.reponumber = reponumber

    def setDB(self,reuse,filename):
        self.dbreuse = reuse
        self.dbfile = filename
