# (c) 2007 by Markus Ullmann
# License: GPL-2


"""module for handling version bumps"""


import os, os.path

from PyQt4 import QtGui, QtCore
from maintainer_helper.gui.DropVersion_ui import Ui_DropVersion

from snakeoil.osutils import pjoin


class drop_version_widget(QtGui.QDialog):
    def __init__(self, parent, version = None):
        QtGui.QDialog.__init__(self, parent)
        self.parent = parent

        self.useriface = Ui_DropVersion()
        self.useriface.setupUi(self)

        (category, package) = self.parent.get_packagetree_entry()
        version = self.parent.get_current_version()

        self.package_path = pjoin(
            self.parent.domain_repo_interface.work_repo_location, category,
            package)

        try:
            testfilename = pjoin(self.package_path, ".testfile")
            testfile = open(testfilename,"w")
            testfile.write(" ")
            os.remove(testfilename)
        except (OSError, IOError):
            QtGui.QMessageBox.critical(self, "Directory not writable",
                "The selected version cannot be deleted, because the " +
                "directory is not writable",
                QtGui.QMessageBox.Ok)
            return

        self.useriface.deleteList.addItem("%s-%s.ebuild" % (package, version))
        self.useriface.deleteList.setCurrentRow(
            self.useriface.deleteList.count()-1)
        self.useriface.deleteList.currentItem().setSelected(True)

        digestfile = pjoin(self.package_path, "files", "digest-%s-%s" % (
            package, version))
        if self.parent.useriface.removeFiles.isChecked():
            for root, dirs, files in os.walk(pjoin(self.package_path,"files")):
                for ignoredir in ('CVS','.svn'):
                    if ignoredir in dirs:
                        dirs.remove(ignoredir)
                for file in files:
                    if file.startswith("digest-"):
                        if file == "digest-%s-%s" % (package, version):
                            self.useriface.deleteList.addItem(pjoin(root[len(self.package_path)+1:],file))
                    else:
                        self.useriface.deleteList.addItem(pjoin(root[len(self.package_path)+1:],file))
            for item in self.useriface.deleteList.findItems(digestfile[len(
                self.package_path)+1:],QtCore.Qt.MatchExactly):
                item.setSelected(True)
        else:
            if os.path.exists(digestfile):
                self.useriface.deleteList.addItem(digestfile[len(
                    self.package_path)+1:])
                self.useriface.deleteList.setCurrentRow(
                    self.useriface.deleteList.count()-1)
                self.useriface.deleteList.currentItem().setSelected(True)

        self.connect(self,
            QtCore.SIGNAL("finished(int)"), self.dialog_closed)

        self.setModal(True)
        self.show()

    def dialog_closed(self, result):
        """dialog finished now, should we bump?

        @param result: return code of QDialog.closed()
        """

        if not result:
            return

        filetextlist = ""
        for item in self.useriface.deleteList.selectedItems():
            filetextlist += item.text() + "\n"

        you_sure_result = QtGui.QMessageBox.warning(self,
            "Really kill these files?",
            "Are you sure to kill these files:\n\n" + filetextlist,
            QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)

        if you_sure_result == QtGui.QMessageBox.No:
            return

        for item in self.useriface.deleteList.selectedItems():
            os.remove(pjoin(self.package_path, str(item.text())))

        self.parent.domain_repo_interface.refresh()
        self.parent.refresh_keywords()
