# (c) 2007 by Markus Ullmann
# License: GPL-2


"""module for handling version bumps"""


import os, os.path, shutil

from PyQt4 import QtGui, QtCore
from maintainer_helper.gui.VersionBump_ui import Ui_VersionBumpDialog

from maintainer_helper.backend import new_revision, domain_repo_interface
from maintainer_helper.backend.tasks import keyword, edit

from snakeoil.osutils import pjoin


class version_bump_widget(QtGui.QDialog):
    def __init__(self, parent, version = None):
        QtGui.QDialog.__init__(self, parent)
        self.parent = parent

        self.useriface = Ui_VersionBumpDialog()
        self.useriface.setupUi(self)

        self.connect(self,
            QtCore.SIGNAL("finished(int)"), self.dialog_closed)

        # initial domain list
        source = self.parent.useriface.domainSelect
        target = self.useriface.targetDomain
        for i in range(0, source.count()):
            target.addItem(source.itemText(i))
        target.setCurrentIndex(target.findText(source.currentText()))

        # initial repo list
        source = self.parent.useriface.repoSelect
        target = self.useriface.targetRepo
        for i in range(0, source.count()):
            target.addItem(source.itemText(i))
        target.setCurrentIndex(target.findText(source.currentText()))

        self.packagetree_entry = self.parent.get_packagetree_entry()
        self.setWindowTitle("Version bump for %s/%s" % self.packagetree_entry)

        # all current versions of said package
        source = self.parent.useriface.keywordBox
        target = self.useriface.oldVersionBox
        for i in range(0, source.rowCount()):
            target.addItem(source.verticalHeaderItem(i).text())
        target.setCurrentIndex(target.count()-1)
        self.useriface.newVersion.setText(
            new_revision(str(target.currentText())))

        if version is not None:
            target.setCurrentIndex(
                target.findText(version))

        self.domain_repo_interface = domain_repo_interface()
        self.domain_repo_interface.set_domain_by_name(
            str(self.useriface.targetDomain.currentText()))

        self.setModal(True)
        self.show()

        self.connect(self.useriface.targetDomain,
            QtCore.SIGNAL("currentIndexChanged(int)"),
            self.domain_select_changed)
        self.connect(self.useriface.targetRepo,
            QtCore.SIGNAL("currentIndexChanged(int)"), self.repo_select_changed)

        self.repo_select_changed(self.useriface.targetRepo.currentIndex())

    def domain_select_changed(self, newindex):
        self.disconnect(self.useriface.targetRepo,
            QtCore.SIGNAL("currentIndexChanged(int)"), self.repo_select_changed)
        self.domain_repo_interface.set_domain_by_name(
            str(self.useriface.targetDomain.currentText()))

        if self.useriface.targetRepo.count() > 0:
            self.useriface.targetRepo.clear()

        repolist = self.domain_repo_interface.get_repolist_ids()
        self.useriface.targetRepo.addItems(repolist)

        self.connect(self.useriface.targetRepo,
            QtCore.SIGNAL("currentIndexChanged(int)"), self.repo_select_changed)

        self.repo_select_changed(0)

    def repo_select_changed(self, newindex):
        selected_repo = str(self.useriface.targetRepo.currentText())
        self.domain_repo_interface.set_work_repo_by_id(selected_repo)
        (category, package) = self.packagetree_entry
        try:
            testfilename = pjoin(self.domain_repo_interface.work_repo_location,
                category, package, ".testfile")
            testfile = open(testfilename,"w")
            testfile.write(" ")
            os.remove(testfilename)
            self.useriface.notWritableLabel.hide()
            self.useriface.buttonBox.clear()
            self.useriface.buttonBox.setStandardButtons(
                QtGui.QDialogButtonBox.Cancel|
                QtGui.QDialogButtonBox.Ok)
        except (OSError, IOError):
            self.useriface.notWritableLabel.show()
            self.useriface.buttonBox.clear()
            self.useriface.buttonBox.setStandardButtons(
                QtGui.QDialogButtonBox.Cancel)

    def dialog_closed(self, result):
        """dialog finished now, should we bump?

        @param result: return code of QDialog.closed()
        """

        if result != True:
            return
        if self.useriface.useOldEbuild.isChecked() == True:
            self.use_old_ebuild()
        elif self.useriface.importFromBugs.isChecked() == True:
            self.import_from_bugs()
        else:
            self.new_ebuild()
        self.parent.domain_repo_interface.refresh()
        self.parent.refresh_keywords()

    def use_old_ebuild(self):
        """we're going to use an old ebuild"""

        (category, package) = self.packagetree_entry
        old_version = str(self.useriface.oldVersionBox.currentText(
            )).encode('ascii')
        new_version = str(self.useriface.newVersion.text()).encode('ascii')

        target_dir = pjoin(
            self.domain_repo_interface.work_repo_location,
            category, package)
        new_ebuild_filename = pjoin(target_dir,
            package + "-" + new_version + ".ebuild")

        shutil.copy(pjoin(target_dir, package + "-" + old_version + ".ebuild"),
            new_ebuild_filename)

        self.manage_keywords(new_ebuild_filename)
        if self.useriface.editAfterCopy.isChecked():
            edit(new_ebuild_filename)
        return

    def import_from_bugs(self):
        """import ebuild from bugzie"""

        return

    def new_ebuild(self):
        """new ebuild from scratch"""

        (category, package) = self.packagetree_entry
        new_version = str(self.useriface.newVersion.text()).encode('ascii')

        target_dir = pjoin(
            self.domain_repo_interface.work_repo_location,
            category, package)
        new_ebuild_filename = pjoin(target_dir,
            package + "-" + new_version + ".ebuild")

        shutil.copy(pjoin(self.parent.configdir, "skel.ebuild"),
            new_ebuild_filename)

        self.useriface.dropKeywords.setChecked(True)
        self.useriface.dropKeywordsKeepArch.setChecked(True)

        self.manage_keywords(new_ebuild_filename)
        edit(new_ebuild_filename)

        return

    def manage_keywords(self, ebuild_filename):
        """handle keywording for new ebuild

        @param ebuild_filename: the file to handle keywords for
        """

        if self.useriface.markTesting.isChecked():
            keyword(ebuild_filename, "testing")
        elif self.useriface.markStable.isChecked():
            keyword(ebuild_filename, "stable")
        else:
            # drop keywords
            if self.useriface.dropKeywordsKeepArch.isChecked():
                keyword(ebuild_filename, "drop")
                keyword(ebuild_filename, "~x86")
            else:
                keyword(ebuild_filename, "drop")
        return
